@@ -2,10 +2,10 @@
 ;;
 ;; "sumibi.el" is a client for Sumibi server.
 ;;
-;;   Copyright (C) 2002,2003,2004,2005 Kiyoka Nishyama
+;;   Copyright (C) 2002,2003,2004,2005 Kiyoka Nishiyama
 ;;   This program was derived from yc.el-4.0.13(auther: knak)
 ;;
-;;     $Date: 2005/10/24 15:31:34 $
+;;     $Date: 2005/11/12 04:02:59 $
 ;;
 ;; This file is part of Sumibi
 ;;
@@ -115,6 +115,22 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   :type 'boolean
   :group 'sumibi)
 
+(defcustom sumibi-realtime-guide-running-seconds 60
+  "$B%j%"%k%?%$%`%,%$%II=<($N7QB3;~4V(B($BIC?t(B)$B!&%<%m$G%,%$%II=<(5!G=$,L58z$K$J$j$^$9!#(B"
+  :type  'integer
+  :group 'sumibi)
+
+(defcustom sumibi-realtime-guide-interval  0.5
+  "$B%j%"%k%?%$%`%,%$%II=<($r99?7$9$k;~4V4V3V(B"
+  :type  'integer
+  :group 'sumibi)
+
+
+(defface sumibi-guide-face
+  '((((class color) (background light)) (:background "#E0E0E0" :foreground "#F03030")))
+  "$B%j%"%k%?%$%`%,%$%I$N%U%'%$%9(B($BAu>~!"?'$J$I$N;XDj(B)"
+  :group 'sumibi)
+
 
 (defvar sumibi-mode nil             "$B4A;zJQ49%H%0%kJQ?t(B")
 (defvar sumibi-mode-line-string     " Sumibi")
@@ -149,6 +165,320 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 (defvar sumibi-select-mode-end-hook nil)
 
 
+(defvar sumibi-hiragana->katakana-table
+  (mapcar
+   (lambda (c)
+     (cons
+      (char-to-string c)
+      (char-to-string
+       (+ c
+	  (- 
+	   (string-to-char "$B%"(B")
+	   (string-to-char "$B$"(B"))))))
+   (string-to-list
+    (concat 
+     "$B$"$$$&$($*(B"
+     "$B$!$#$%$'$)(B"
+     "$B$+$-$/$1$3(B"
+     "$B$,$.$0$2$4(B"
+     "$B$5$7$9$;$=(B"
+     "$B$6$8$:$<$>(B"
+     "$B$?$A$D$F$H(B"
+     "$B$@$E$E$G$I(B"
+     "$B$J$K$L$M$N(B"
+     "$B$O$R$U$X$[(B"
+     "$B$P$S$V$Y$\(B"
+     "$B$Q$T$W$Z$](B"
+     "$B$^$_$`$a$b(B"
+     "$B$d$f$h(B"
+     "$B$c$e$g(B"
+     "$B$i$j$k$l$m(B"
+     "$B$o$r(B"
+     "$B$C$s(B"))))
+
+
+(defvar sumibi-roman->kana-table
+  '(("kkya" . "$B$C$-$c(B")
+    ("kkyu" . "$B$C$-$e(B")
+    ("kkyo" . "$B$C$-$g(B")
+    ("ggya" . "$B$C$.$c(B")
+    ("ggyu" . "$B$C$.$e(B")
+    ("ggyo" . "$B$C$.$g(B")
+    ("sshi" . "$B$C$7(B")
+    ("ssha" . "$B$C$7$c(B")
+    ("sshu" . "$B$C$7$e(B")
+    ("sshe" . "$B$C$7$'(B")
+    ("ssho" . "$B$C$7$g(B")
+    ("cchi" . "$B$C$A(B")
+    ("ccha" . "$B$C$A$c(B")
+    ("cchu" . "$B$C$A$e(B")
+    ("cche" . "$B$C$A$'(B")
+    ("ccho" . "$B$C$A$g(B")
+    ("ddya" . "$B$C$B$c(B")
+    ("ddyu" . "$B$C$B$e(B")
+    ("ddye" . "$B$C$B$'(B")
+    ("ddyo" . "$B$C$B$g(B")
+    ("ttsu" . "$B$C$D(B")
+    ("hhya" . "$B$C$R$c(B")
+    ("hhyu" . "$B$C$R$e(B")
+    ("hhyo" . "$B$C$R$g(B")
+    ("bbya" . "$B$C$S$c(B")
+    ("bbyu" . "$B$C$S$e(B")
+    ("bbyo" . "$B$C$S$g(B")
+    ("ppya" . "$B$C$T$c(B")
+    ("ppyu" . "$B$C$T$e(B")
+    ("ppyo" . "$B$C$T$g(B")
+    ("rrya" . "$B$C$j$c(B")
+    ("rryu" . "$B$C$j$e(B")
+    ("rryo" . "$B$C$j$g(B")
+    ("ddyi" . "$B$C$G$#(B")
+    ("ddhi" . "$B$C$G$#(B")
+    ("xtsu" . "$B$C(B")
+    ("ttya" . "$B$C$A$c(B")
+    ("ttyi" . "$B$C$A(B")
+    ("ttyu" . "$B$C$A$e(B")
+    ("ttye" . "$B$C$A$'(B")
+    ("ttyo" . "$B$C$A$g(B")
+    ("kya" . "$B$-$c(B")
+    ("kyu" . "$B$-$e(B")
+    ("kyo" . "$B$-$g(B")
+    ("gya" . "$B$.$c(B")
+    ("gyu" . "$B$.$e(B")
+    ("gyo" . "$B$.$g(B")
+    ("shi" . "$B$7(B")
+    ("sha" . "$B$7$c(B")
+    ("shu" . "$B$7$e(B")
+    ("she" . "$B$7$'(B")
+    ("sho" . "$B$7$g(B")
+    ("chi" . "$B$A(B")
+    ("cha" . "$B$A$c(B")
+    ("chu" . "$B$A$e(B")
+    ("che" . "$B$A$'(B")
+    ("cho" . "$B$A$g(B")
+    ("dya" . "$B$B$c(B")
+    ("dyu" . "$B$B$e(B")
+    ("dye" . "$B$B$'(B")
+    ("dyo" . "$B$B$g(B")
+    ("vvu" . "$B$C$&!+(B")
+    ("vva" . "$B$C$&!+$!(B")
+    ("vvi" . "$B$C$&!+$#(B")
+    ("vve" . "$B$C$&!+$'(B")
+    ("vvo" . "$B$C$&!+$)(B")
+    ("kka" . "$B$C$+(B")
+    ("gga" . "$B$C$,(B")
+    ("kki" . "$B$C$-(B")
+    ("ggi" . "$B$C$.(B")
+    ("kku" . "$B$C$/(B")
+    ("ggu" . "$B$C$0(B")
+    ("kke" . "$B$C$1(B")
+    ("gge" . "$B$C$2(B")
+    ("kko" . "$B$C$3(B")
+    ("ggo" . "$B$C$4(B")
+    ("ssa" . "$B$C$5(B")
+    ("zza" . "$B$C$6(B")
+    ("jji" . "$B$C$8(B")
+    ("jja" . "$B$C$8$c(B")
+    ("jju" . "$B$C$8$e(B")
+    ("jje" . "$B$C$8$'(B")
+    ("jjo" . "$B$C$8$g(B")
+    ("ssu" . "$B$C$9(B")
+    ("zzu" . "$B$C$:(B")
+    ("sse" . "$B$C$;(B")
+    ("zze" . "$B$C$<(B")
+    ("sso" . "$B$C$=(B")
+    ("zzo" . "$B$C$>(B")
+    ("tta" . "$B$C$?(B")
+    ("dda" . "$B$C$@(B")
+    ("ddi" . "$B$C$B(B")
+    ("ddu" . "$B$C$E(B")
+    ("tte" . "$B$C$F(B")
+    ("dde" . "$B$C$G(B")
+    ("tto" . "$B$C$H(B")
+    ("ddo" . "$B$C$I(B")
+    ("hha" . "$B$C$O(B")
+    ("bba" . "$B$C$P(B")
+    ("ppa" . "$B$C$Q(B")
+    ("hhi" . "$B$C$R(B")
+    ("bbi" . "$B$C$S(B")
+    ("ppi" . "$B$C$T(B")
+    ("ffu" . "$B$C$U(B")
+    ("ffa" . "$B$C$U$!(B")
+    ("ffi" . "$B$C$U$#(B")
+    ("ffe" . "$B$C$U$'(B")
+    ("ffo" . "$B$C$U$)(B")
+    ("bbu" . "$B$C$V(B")
+    ("ppu" . "$B$C$W(B")
+    ("hhe" . "$B$C$X(B")
+    ("bbe" . "$B$C$Y(B")
+    ("ppe" . "$B$C$Z(B")
+    ("hho" . "$B$C$[(B")
+    ("bbo" . "$B$C$\(B")
+    ("ppo" . "$B$C$](B")
+    ("yya" . "$B$C$d(B")
+    ("yyu" . "$B$C$f(B")
+    ("yyo" . "$B$C$h(B")
+    ("rra" . "$B$C$i(B")
+    ("rri" . "$B$C$j(B")
+    ("rru" . "$B$C$k(B")
+    ("rre" . "$B$C$l(B")
+    ("rro" . "$B$C$m(B")
+    ("tsu" . "$B$D(B")
+    ("nya" . "$B$K$c(B")
+    ("nyu" . "$B$K$e(B")
+    ("nyo" . "$B$K$g(B")
+    ("hya" . "$B$R$c(B")
+    ("hyu" . "$B$R$e(B")
+    ("hyo" . "$B$R$g(B")
+    ("bya" . "$B$S$c(B")
+    ("byu" . "$B$S$e(B")
+    ("byo" . "$B$S$g(B")
+    ("pya" . "$B$T$c(B")
+    ("pyu" . "$B$T$e(B")
+    ("pyo" . "$B$T$g(B")
+    ("mya" . "$B$_$c(B")
+    ("myu" . "$B$_$e(B")
+    ("myo" . "$B$_$g(B")
+    ("xya" . "$B$c(B")
+    ("xyu" . "$B$e(B")
+    ("xyo" . "$B$g(B")
+    ("rya" . "$B$j$c(B")
+    ("ryu" . "$B$j$e(B")
+    ("ryo" . "$B$j$g(B")
+    ("xwa" . "$B$n(B")
+    ("dyi" . "$B$G$#(B")
+    ("thi" . "$B$F$#(B")
+    ("hhu" . "$B$C$U(B")
+    ("shu" . "$B$7$e(B")
+    ("chu" . "$B$A$e(B")
+    ("sya" . "$B$7$c(B")
+    ("syu" . "$B$7$e(B")
+    ("sye" . "$B$7$'(B")
+    ("syo" . "$B$7$g(B")
+    ("jya" . "$B$8$c(B")
+    ("jyu" . "$B$8$e(B")
+    ("jye" . "$B$8$'(B")
+    ("jyo" . "$B$8$g(B")
+    ("zya" . "$B$8$c(B")
+    ("zyu" . "$B$8$e(B")
+    ("zye" . "$B$8$'(B")
+    ("zyo" . "$B$8$g(B")
+    ("tya" . "$B$A$c(B")
+    ("tyi" . "$B$A(B")
+    ("tyu" . "$B$A$e(B")
+    ("tye" . "$B$A$'(B")
+    ("tyo" . "$B$A$g(B")
+    ("dhi" . "$B$G$#(B")
+    ("xtu" . "$B$C(B")
+    ("xa" . "$B$!(B")
+    ("xi" . "$B$#(B")
+    ("xu" . "$B$%(B")
+    ("vu" . "$B$&!+(B")
+    ("va" . "$B$&!+$!(B")
+    ("vi" . "$B$&!+$#(B")
+    ("ve" . "$B$&!+$'(B")
+    ("vo" . "$B$&!+$)(B")
+    ("xe" . "$B$'(B")
+    ("xo" . "$B$)(B")
+    ("ka" . "$B$+(B")
+    ("ga" . "$B$,(B")
+    ("ki" . "$B$-(B")
+    ("gi" . "$B$.(B")
+    ("ku" . "$B$/(B")
+    ("gu" . "$B$0(B")
+    ("ke" . "$B$1(B")
+    ("ge" . "$B$2(B")
+    ("ko" . "$B$3(B")
+    ("go" . "$B$4(B")
+    ("sa" . "$B$5(B")
+    ("za" . "$B$6(B")
+    ("ji" . "$B$8(B")
+    ("ja" . "$B$8$c(B")
+    ("ju" . "$B$8$e(B")
+    ("je" . "$B$8$'(B")
+    ("jo" . "$B$8$g(B")
+    ("su" . "$B$9(B")
+    ("zu" . "$B$:(B")
+    ("se" . "$B$;(B")
+    ("ze" . "$B$<(B")
+    ("so" . "$B$=(B")
+    ("zo" . "$B$>(B")
+    ("ta" . "$B$?(B")
+    ("da" . "$B$@(B")
+    ("di" . "$B$B(B")
+    ("tt" . "$B$C(B")
+    ("du" . "$B$E(B")
+    ("te" . "$B$F(B")
+    ("de" . "$B$G(B")
+    ("to" . "$B$H(B")
+    ("do" . "$B$I(B")
+    ("na" . "$B$J(B")
+    ("ni" . "$B$K(B")
+    ("nu" . "$B$L(B")
+    ("ne" . "$B$M(B")
+    ("no" . "$B$N(B")
+    ("ha" . "$B$O(B")
+    ("ba" . "$B$P(B")
+    ("pa" . "$B$Q(B")
+    ("hi" . "$B$R(B")
+    ("bi" . "$B$S(B")
+    ("pi" . "$B$T(B")
+    ("fu" . "$B$U(B")
+    ("fa" . "$B$U$!(B")
+    ("fi" . "$B$U$#(B")
+    ("fe" . "$B$U$'(B")
+    ("fo" . "$B$U$)(B")
+    ("bu" . "$B$V(B")
+    ("pu" . "$B$W(B")
+    ("he" . "$B$X(B")
+    ("be" . "$B$Y(B")
+    ("pe" . "$B$Z(B")
+    ("ho" . "$B$[(B")
+    ("bo" . "$B$\(B")
+    ("po" . "$B$](B")
+    ("ma" . "$B$^(B")
+    ("mi" . "$B$_(B")
+    ("mu" . "$B$`(B")
+    ("me" . "$B$a(B")
+    ("mo" . "$B$b(B")
+    ("ya" . "$B$d(B")
+    ("yu" . "$B$f(B")
+    ("yo" . "$B$h(B")
+    ("ra" . "$B$i(B")
+    ("ri" . "$B$j(B")
+    ("ru" . "$B$k(B")
+    ("re" . "$B$l(B")
+    ("ro" . "$B$m(B")
+    ("wa" . "$B$o(B")
+    ("wi" . "$B$p(B")
+    ("we" . "$B$q(B")
+    ("wo" . "$B$r(B")
+    ("n'" . "$B$s(B")
+    ("nn" . "$B$s(B")
+    ("ca" . "$B$+(B")
+    ("ci" . "$B$-(B")
+    ("cu" . "$B$/(B")
+    ("ce" . "$B$1(B")
+    ("co" . "$B$3(B")
+    ("si" . "$B$7(B")
+    ("ti" . "$B$A(B")
+    ("hu" . "$B$U(B")
+    ("tu" . "$B$D(B")
+    ("zi" . "$B$8(B")
+    ("la" . "$B$!(B")
+    ("li" . "$B$#(B")
+    ("lu" . "$B$%(B")
+    ("le" . "$B$'(B")
+    ("lo" . "$B$)(B")
+    ("a" . "$B$"(B")
+    ("i" . "$B$$(B")
+    ("u" . "$B$&(B")
+    ("e" . "$B$((B")
+    ("o" . "$B$*(B")
+    ("n" . "$B$s(B")
+    ("-" . "$B!<(B")
+    ("^" . "$B!<(B")))
+
+
 ;;--- $B%G%P%C%0%a%C%;!<%8=PNO(B
 (defvar sumibi-debug nil)		; $B%G%P%C%0%U%i%0(B
 (defun sumibi-debug-print (string)
@@ -179,6 +509,9 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 (defvar sumibi-repeat 0)		; $B7+$jJV$72s?t(B
 (defvar sumibi-marker-list '())		; $BJ8@a3+;O!"=*N;0LCV%j%9%H(B: $B<!$N$h$&$J7A<0(B ( ( 1 . 2 ) ( 5 . 7 ) ... ) 
 (defvar sumibi-timer    nil)            ; $B%$%s%?!<%P%k%?%$%^!<7?JQ?t(B
+(defvar sumibi-timer-rest  0)           ; $B$"$H2?2s8F=P$5$l$?$i!"%$%s%?!<%P%k%?%$%^$N8F=P$r;_$a$k$+(B
+(defvar sumibi-guide-overlay   nil)     ; $B%j%"%k%?%$%`%,%$%I$K;HMQ$9$k%*!<%P!<%l%$(B
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; $BI=<(7O4X?t72(B
@@ -216,13 +549,6 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	      (lambda ()
 		(delete-file sumibi-server-cert-file)))
 
-    ;; $B%?%$%^!<%$%Y%s%H4X?t$NEPO?(B
-    ;;    (setq sumibi-timer
-    ;;	  (run-at-time 1 1
-    ;;		       '(lambda ()
-    ;;			  (message "sumibi-time-handler"))))
-
-
     ;; $B=i4|2=40N;(B
     (setq sumibi-init t)))
 
@@ -698,8 +1024,34 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 $B!&%+!<%=%k$+$i9TF,J}8~$K%m!<%^;zNs$,B3$/HO0O$G%m!<%^;z4A;zJQ49$r9T$&!#(B"
   (interactive)
 ;  (print last-command)			; DEBUG
+
+  (cond 
+   ;; $B%?%$%^!<%$%Y%s%H$r@_Dj$7$J$$>r7o(B
+   ((or
+     sumibi-timer
+     (> 1 sumibi-realtime-guide-running-seconds)
+     ))
+   (t
+    ;; $B%?%$%^!<%$%Y%s%H4X?t$NEPO?(B
+    (progn
+      (let 
+	  ((ov-point
+	    (save-excursion
+	      (forward-line 1)
+	      (point))))
+	  (setq sumibi-guide-overlay
+			(make-overlay ov-point ov-point (current-buffer))))
+      (setq sumibi-timer
+			(run-at-time 0.1 sumibi-realtime-guide-interval
+						 'sumibi-realtime-guide)))))
+
+  ;; $B%,%$%II=<(7QB32s?t$N99?7(B
+  (when (< 0 sumibi-realtime-guide-running-seconds)
+    (setq sumibi-timer-rest  
+	  (/ sumibi-realtime-guide-running-seconds
+	     sumibi-realtime-guide-interval)))
+
   (cond
-   
    (sumibi-select-mode
     ;; $BJQ49Cf$K8F=P$5$l$?$i!"8uJdA*Br%b!<%I$K0\9T$9$k!#(B
     (funcall (lookup-key sumibi-select-mode-map sumibi-rK-trans-key)))
@@ -847,6 +1199,40 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 	     (point))
 	  result)))))
 
+;;;
+;;; $B%m!<%+%k$N(BEmacsLisp$B$@$1$GJQ49$9$k=hM}(B
+;;;
+;; a-list $B$r;H$C$F(B str $B$N@hF,$K3:Ev$9$kJ8;zNs$,$"$k$+D4$Y$k(B
+(defun romkan-scan-token (a-list str)
+  (let 
+      ((result     (substring str 0 1))
+       (rest       (substring str 1 (length str)))
+       (done       nil))
+
+    (mapcar
+     (lambda (x)
+       (if (and 
+	    (string-match (concat "^" (car x)) str)
+	    (not done))
+	   (progn
+	     (setq done t)
+	     (setq result (cdr x))
+	     (setq rest   (substring str (length (car x)))))))
+     a-list)
+    (cons result rest)))
+
+
+;; $B$+$J(B<->$B%m!<%^;zJQ49$9$k(B
+(defun romkan-convert (a-list str)
+  (cond ((= 0 (length str))
+	 "")
+	(t
+	 (let ((ret (romkan-scan-token a-list str)))
+	   (concat
+	    (car ret)
+	    (romkan-convert a-list (cdr ret)))))))
+
+
   
 ;;;
 ;;; with viper
@@ -879,6 +1265,87 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
   (remove-hook 'sumibi-mode-hook 'sumibi-viper-init-function))
 
 
+
+(defun sumibi-realtime-guide ()
+  "$B%j%"%k%?%$%`$GJQ49Cf$N%,%$%I$r=P$9(B
+sumibi-mode$B$,(BON$B$N4VCf8F$S=P$5$l$k2DG=@-$,$"$k!&(B"
+  (cond
+   ((> 1 sumibi-timer-rest)
+    (cancel-timer sumibi-timer)
+    (setq sumibi-timer nil)
+    (delete-overlay sumibi-guide-overlay))
+   (sumibi-guide-overlay
+    ;; $B;D$j2s?t$N%G%/%j%a%s%H(B
+    (setq sumibi-timer-rest (- sumibi-timer-rest 1))
+
+    (let* (
+	   (end (point))
+	   (gap (sumibi-skip-chars-backward))
+	   (prev-line-existp
+	    (not (= (point-at-bol) (point-min))))
+	   (next-line-existp
+	    (not (= (point-at-eol) (point-max))))
+	   (prev-line-point
+	    (when prev-line-existp
+	      (save-excursion
+		(forward-line -1)
+		(point))))
+	   (next-line-point
+	    (when next-line-existp
+	      (save-excursion
+		(forward-line 1)
+		(point))))
+	   (disp-point
+	    (or next-line-point prev-line-point)))
+
+      (if 
+	  (or 
+	   (when (fboundp 'minibufferp)
+	     (minibufferp))
+	   (and
+	    (not next-line-point)
+	    (not prev-line-point))
+	   (= gap 0))
+	  ;; $B>e2<%9%Z!<%9$,L5$$(B $B$^$?$O(B $BJQ49BP>]$,L5$7$J$i%,%$%I$OI=<($7$J$$!#(B
+	  (overlay-put sumibi-guide-overlay 'before-string "")
+	;; $B0UL#$N$"$kF~NO$,8+$D$+$C$?$N$G%,%$%I$rI=<($9$k!#(B
+	(let* (
+	       (b (+ end gap))
+	       (e end)
+	       (str (buffer-substring b e))
+	       (l (split-string str))
+	       (mess
+		(mapconcat
+		 (lambda (x)
+		   (let* ((l (split-string x "\\."))
+			  (method
+			   (when (< 1 (length l))
+			     (cadr l)))
+			  (hira
+			   (romkan-convert sumibi-roman->kana-table
+					   (car l))))
+		     (cond
+		      ((string-match "[a-z]+" hira)
+		       x)
+		      ((not method)
+		       hira)
+		      ((or (string-equal "j" method) (string-equal "h" method))
+		       hira)
+		      ((or (string-equal "e" method) (string-equal "l" method))
+		       (car l))
+		      ((string-equal "k" method)
+		       (romkan-convert sumibi-hiragana->katakana-table
+				       hira))
+		      (t
+		       x))))
+		 l
+		 " ")))
+	  (move-overlay sumibi-guide-overlay 
+			disp-point (min (point-max) (+ disp-point 1)) (current-buffer))
+	  (overlay-put sumibi-guide-overlay 'before-string mess))))
+    (overlay-put sumibi-guide-overlay 'face 'sumibi-guide-face))))
+
+
 ;;;
 ;;; human interface
 ;;;
@@ -978,7 +1445,7 @@ point $B$+$i9TF,J}8~$KF1<o$NJ8;zNs$,B3$/4V$r4A;zJQ49$7$^$9!#(B
 (setq default-input-method "japanese-sumibi")
 
 (defconst sumibi-version
-"0.5.2"
+"0.5.3"
   )
 (defun sumibi-version (&optional arg)
   "$BF~NO%b!<%IJQ99(B"
